ICoWLock

Struct ICoWLock 

pub struct ICoWLock<'lock, ITEM: Sized> { /* private fields */ }
A write-guard which holds new value to which the new values are written. And previous value too. This type of guard is exclusive, so multiple CoW operations CANNOT be performed in parallel which is good for instance update without racing. The readers are also waiting until the changes are commited.

If dropped, no changes will be made.

Implementations

Source

impl<'lock, ITEM: Sized> ICoWLock<'lock, ITEM>

pub fn prev_val(&self) -> &ITEM

Returns a reference to previous value. To access modified version use Deref or DerefMut.

pub fn commit(self)

Commits the changes made in the current guarded instance guarantees that the future readers will read new value and no other writes will be perfomed at the same time.

Returns

Returns nothing