ICoWRead

Struct ICoWRead 

pub struct ICoWRead<'read, ITEM: Sized> { /* private fields */ }
A read only guard. Implements Deref only. The guarded value is valid all the time, even if the inner value was updated. The updated value will not be visible in the current instance until re-read is performed.

Implementations

Source

impl<'read, ITEM: Sized + Debug + Clone> ICoWRead<'read, ITEM>

pub fn clone_copy_into_inner(self) -> ITEM

Consumes the itance and returns the clonned ITEM value.

Source

impl<'read, ITEM: Sized + Debug + Copy> ICoWRead<'read, ITEM>

pub fn copy_into_inner(self) -> ITEM

Consumes the itance and returns the copied ITEM value.

Source

impl<'read, ITEM: Sized> ICoWRead<'read, ITEM>

pub fn weak(&self) -> Weak<ITEM>

Returns the weak reference.

Source

impl<'read, ITEM: Sized + Debug + Copy> ICoWRead<'read, ITEM>

pub fn upgrade_copy(self) -> ICoWCopy<'read, ITEM>

Upgrades from ICoWRead the self to copy ICoWCopy which can be commited to mainstream by using Copy.

Source

impl<'read, ITEM: Sized + Debug + Clone> ICoWRead<'read, ITEM>

pub fn upgrade_clone_copy(self) -> ICoWCopy<'read, ITEM>

Upgrades from ICoWRead the self to copy ICoWCopy which can be commited to mainstream by using Clone.

Source

impl<'read, ITEM: Sized + Debug + Default> ICoWRead<'read, ITEM>

pub fn upgrade_default(self) -> ICoWCopy<'read, ITEM>

Upgrades from ICoWRead the self to copy ICoWCopy which can be commited to mainstream by using Default.