pub struct ICoWWeak<ITEM: Sized> { /* private fields */ }Expand description
A independent from the ICoW instance, which is not binded to the lifetime
of the main instance. The value contains a read-only weak reference to ITEM.
If the weak reference become invalid, then a new copy will be obtained.
It should not be accessed from multiple threads simultaniously!
Implementations