ICoWCopy

Struct ICoWCopy 

pub struct ICoWCopy<'copy, ITEM: Sized> { /* private fields */ }
A write-guard which holds new value to which the new data is written. And previous value too. This type of guard is not exclusive, so multiple CoW operations may be performed in parallel which is not normally needed.

The changes made in the current instance becomes visible for the rest of the threads only after commit.

Implementations

Source

impl<'copy, ITEM: Sized> ICoWCopy<'copy, ITEM>

pub fn prev_val(&self) -> &ITEM

Returns a reference to previous value. To access modified version use Deref or DerefMut.

pub fn commit(self)

Commits the changes made in the guarded variable. A non-blocking function. It will not block the thread completly and returns the [ICoWError::WouldBlock] if attempt to grab the pointer atomically fails in reasonable time. for blocking commit use [Self::commit_blocking].

pub fn into_inner(self) -> ITEM

Drops the instance without commiting changes returning a copy, clone, default, or new i.e what was in the new field of the instance.