pub struct ICoWLock<'lock, ITEM: Sized> { /* private fields */ }Expand description
A write-guard which holds new value to which the new values are written. And previous value too. This type of guard is exclusive, so multiple CoW operations CANNOT be performed in parallel which is good for instance update without racing. The readers are also waiting until the changes are commited.
If dropped, no changes will be made.
Implementations